@@ -61,11 +61,23 @@ static const QString appViewName = QStringLiteral("org.kde.KWin.Effect.WindowVie
 static const QString appViewPath = QStringLiteral("/org/kde/KWin/Effect/WindowView1");
 static const QString &appViewInterface = appViewName;
 
+static constexpr int NoApplications = 2; // kactivitymanager StatsPlugin WhatToRemember.
+
 Backend::Backend(QObject *parent)
     : QObject(parent)
     , m_highlightWindows(false)
     , m_actionGroup(new QActionGroup(this))
+    , m_activityManagerPluginsSettingsWatcher(KConfigWatcher::create(m_activityManagerPluginsSettings.sharedConfig()))
 {
+    connect(m_activityManagerPluginsSettingsWatcher.get(),
+            &KConfigWatcher::configChanged,
+            this,
+            [this](const KConfigGroup &group, const QByteArrayList &names) {
+                if (group.name() == QLatin1String("Plugin-org.kde.ActivityManager.Resources.Scoring")
+                    && names.contains(QByteArrayLiteral("what-to-remember"))) {
+                    m_activityManagerPluginsSettings.load();
+                }
+            });
 }
 
 Backend::~Backend()
@@ -173,6 +185,10 @@ QVariantList Backend::systemSettingsActions(QObject *parent) const
 {
     QVariantList actions;
 
+    if (m_activityManagerPluginsSettings.whatToRemember() == NoApplications) {
+        return actions;
+    }
+
     auto query = AllResources | Agent(QStringLiteral("org.kde.systemsettings")) | HighScoredFirst | Limit(5);
 
     ResultSet results(query);
@@ -311,6 +327,10 @@ QVariantList Backend::recentDocumentActions(const QUrl &launcherUrl, QObject *pa
         return actions;
     }
 
+    if (m_activityManagerPluginsSettings.whatToRemember() == NoApplications) {
+        return actions;
+    }
+
     QUrl desktopEntryUrl = tryDecodeApplicationsUrl(launcherUrl);
 
     if (!desktopEntryUrl.isValid() || !desktopEntryUrl.isLocalFile() || !KDesktopFile::isDesktopFile(desktopEntryUrl.toLocalFile())) {
@@ -440,7 +460,7 @@ void Backend::handleRecentDocumentAction() const
     const QUrl url = action->data().toUrl();
 
     if (desktopPath.isEmpty() || url.isEmpty()) {
-        auto query = UsedResources | Agent(agent) | Type::any() | Activity::current() | Url::file();
+        auto query = UsedResources | Agent(agent) | Type::any() | Activity::current();
 
         KAStats::forgetResources(query);
 
