@@ -26,12 +26,21 @@ class InputDevice : public QObject
     Q_PROPERTY(bool supportsLeftHanded READ supportsLeftHanded CONSTANT)
     Q_PROPERTY(bool leftHanded READ isLeftHanded WRITE setLeftHanded NOTIFY leftHandedChanged)
     Q_PROPERTY(bool supportsCalibrationMatrix READ supportsCalibrationMatrix CONSTANT)
+    Q_PROPERTY(bool calibrationMatrixIsDefault READ calibrationMatrixIsDefault NOTIFY calibrationMatrixChanged)
 
     Q_PROPERTY(bool supportsOrientation READ supportsOrientation CONSTANT)
     Q_PROPERTY(int orientation READ orientation WRITE setOrientation NOTIFY orientationChanged)
     Q_PROPERTY(QString outputName READ outputName WRITE setOutputName NOTIFY outputNameChanged)
     Q_PROPERTY(QRectF outputArea READ outputArea WRITE setOutputArea NOTIFY outputAreaChanged)
+    Q_PROPERTY(QRectF inputArea READ inputArea WRITE setInputArea NOTIFY inputAreaChanged)
+    Q_PROPERTY(bool supportsInputArea READ supportsInputArea CONSTANT)
     Q_PROPERTY(bool mapToWorkspace READ isMapToWorkspace WRITE setMapToWorkspace NOTIFY mapToWorkspaceChanged)
+    Q_PROPERTY(QString pressureCurve READ pressureCurve WRITE setPressureCurve NOTIFY pressureCurveChanged)
+    Q_PROPERTY(bool pressureCurveIsDefault READ pressureCurveIsDefault NOTIFY pressureCurveChanged)
+    Q_PROPERTY(quint32 tabletPadButtonCount READ tabletPadButtonCount CONSTANT)
+    Q_PROPERTY(double pressureRangeMin READ pressureRangeMin WRITE setPressureRangeMin NOTIFY pressureRangeMinChanged)
+    Q_PROPERTY(double pressureRangeMax READ pressureRangeMax WRITE setPressureRangeMax NOTIFY pressureRangeMaxChanged)
+    Q_PROPERTY(bool supportsPressureRange READ supportsPressureRange CONSTANT)
 
 public:
     InputDevice(const QString &dbusName, QObject *parent);
@@ -87,9 +96,19 @@ public:
     }
     void setOutputArea(const QRectF &outputArea);
 
+    QRectF inputArea() const
+    {
+        return m_inputArea.value();
+    }
+    void setInputArea(const QRectF &inputArea);
+    bool supportsInputArea() const
+    {
+        return m_inputArea.isSupported();
+    }
+
     bool supportsCalibrationMatrix() const
     {
-        return m_supportsCalibrationMatrix.value();
+        return m_calibrationMatrix.isSupported();
     }
 
     bool isEnabled() const
@@ -99,19 +118,22 @@ public:
 
     QMatrix4x4 defaultCalibrationMatrix() const
     {
-        return m_iface->property("defaultCalibrationMatrix").value<QMatrix4x4>();
+        return deserializeMatrix(m_calibrationMatrix.defaultValue());
     }
 
     QMatrix4x4 calibrationMatrix() const
     {
-        return m_iface->property("calibrationMatrix").value<QMatrix4x4>();
+        return deserializeMatrix(m_calibrationMatrix.value());
     }
 
-    void setCalibrationMatrix(const QMatrix4x4 &matrix) const
+    void setCalibrationMatrix(const QMatrix4x4 &matrix)
     {
-        // TODO: why does this not with our existing interface...?
-        QDBusInterface iface(QStringLiteral("org.kde.KWin"), m_iface->path());
-        iface.setProperty("calibrationMatrix", matrix);
+        m_calibrationMatrix.set(serializeMatrix(matrix));
+    }
+
+    bool calibrationMatrixIsDefault() const
+    {
+        return m_calibrationMatrix.isDefaults();
     }
 
     void setEnabled(bool enabled);
@@ -122,6 +144,56 @@ public:
     }
     void setMapToWorkspace(bool mapToWorkspace);
 
+    QString pressureCurve() const
+    {
+        return m_pressureCurve.value();
+    }
+    void setPressureCurve(const QString &curve);
+    bool pressureCurveIsDefault() const;
+
+    void setPressureRangeMin(double min)
+    {
+        m_pressureRangeMin.set(min);
+    }
+    double pressureRangeMin() const
+    {
+        return m_pressureRangeMin.value();
+    }
+
+    void setPressureRangeMax(double max)
+    {
+        m_pressureRangeMax.set(max);
+    }
+    double pressureRangeMax() const
+    {
+        return m_pressureRangeMax.value();
+    }
+
+    bool supportsPressureRange() const
+    {
+        return m_pressureRangeMin.isSupported() && m_pressureRangeMax.isSupported();
+    }
+
+    quint32 tabletPadButtonCount() const
+    {
+        return m_tabletPadButtonCount.value();
+    }
+
+    QString deviceGroup() const
+    {
+        return m_deviceGroup.value();
+    }
+
+    bool tabletPad() const
+    {
+        return m_tabletPad.value();
+    }
+
+    bool tabletTool() const
+    {
+        return m_tabletTool.value();
+    }
+
 Q_SIGNALS:
     void needsSaveChanged();
 
@@ -131,6 +203,11 @@ Q_SIGNALS:
     void outputAreaChanged();
     void enabledChanged();
     void mapToWorkspaceChanged();
+    void pressureCurveChanged();
+    void inputAreaChanged();
+    void pressureRangeMinChanged();
+    void pressureRangeMaxChanged();
+    void calibrationMatrixChanged();
 
 private:
     template<typename T>
@@ -225,6 +302,9 @@ private:
         mutable std::optional<T> m_value;
     };
 
+    static QString serializeMatrix(const QMatrix4x4 &matrix);
+    static QMatrix4x4 deserializeMatrix(const QString &matrix);
+
     //
     // general
     Prop<QString> m_name = Prop<QString>(this, "name");
@@ -246,12 +326,40 @@ private:
                                              &OrgKdeKWinInputDeviceInterface::defaultOutputArea,
                                              &OrgKdeKWinInputDeviceInterface::supportsOutputArea,
                                              &InputDevice::outputAreaChanged);
+    Prop<QRectF> m_inputArea = Prop<QRectF>(this,
+                                            "inputArea",
+                                            &OrgKdeKWinInputDeviceInterface::defaultInputArea,
+                                            &OrgKdeKWinInputDeviceInterface::supportsInputArea,
+                                            &InputDevice::inputAreaChanged);
 
     Prop<bool> m_mapToWorkspace =
         Prop<bool>(this, "mapToWorkspace", &OrgKdeKWinInputDeviceInterface::defaultMapToWorkspace, nullptr, &InputDevice::mapToWorkspaceChanged);
 
-    Prop<bool> m_supportsCalibrationMatrix =
-        Prop<bool>(this, "supportsCalibrationMatrix", nullptr, &OrgKdeKWinInputDeviceInterface::supportsCalibrationMatrix, nullptr);
+    Prop<QString> m_calibrationMatrix = Prop<QString>(this,
+                                                      "calibrationMatrix",
+                                                      &OrgKdeKWinInputDeviceInterface::defaultCalibrationMatrix,
+                                                      &OrgKdeKWinInputDeviceInterface::supportsCalibrationMatrix,
+                                                      &InputDevice::calibrationMatrixChanged);
+
+    Prop<QString> m_pressureCurve =
+        Prop<QString>(this, "pressureCurve", &OrgKdeKWinInputDeviceInterface::defaultPressureCurve, nullptr, &InputDevice::pressureCurveChanged);
+
+    Prop<quint32> m_tabletPadButtonCount = Prop<quint32>(this, "tabletPadButtonCount");
+
+    Prop<double> m_pressureRangeMin = Prop<double>(this,
+                                                   "pressureRangeMin",
+                                                   &OrgKdeKWinInputDeviceInterface::defaultPressureRangeMin,
+                                                   &OrgKdeKWinInputDeviceInterface::supportsPressureRange,
+                                                   &InputDevice::pressureRangeMinChanged);
+    Prop<double> m_pressureRangeMax = Prop<double>(this,
+                                                   "pressureRangeMax",
+                                                   &OrgKdeKWinInputDeviceInterface::defaultPressureRangeMax,
+                                                   &OrgKdeKWinInputDeviceInterface::supportsPressureRange,
+                                                   &InputDevice::pressureRangeMaxChanged);
+
+    Prop<QString> m_deviceGroup = Prop<QString>(this, "deviceGroupId");
+    Prop<bool> m_tabletPad = Prop<bool>(this, "tabletPad");
+    Prop<bool> m_tabletTool = Prop<bool>(this, "tabletTool");
 
     std::unique_ptr<OrgKdeKWinInputDeviceInterface> m_iface;
 };
